<?php
/**
 * @package Nicepage Website Builder
 * @author Nicepage https://www.nicepage.com
 * @copyright Copyright (c) 2016 - 2019 Nicepage
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 or later
 */
defined('_JEXEC') or die;

$comDir = JPATH_ADMINISTRATOR . '/components/com_nicepage';
JLoader::register('NicepageHelpersNicepage', $comDir . '/helpers/nicepage.php');
JLoader::register('Nicepage_Theme_Nicepage', $comDir . '/library/theme.php');
require $comDir . '/helpers/modules.php';

$app = JFactory::getApplication();
$doc = JFactory::getDocument();

$themeData = $app->get('themeData');

$context      = $themeData['context'];
$row          = $themeData['row'];
$rowParams    = $themeData['params'];
$editLinkHtml = $themeData['editLinkHtml'];

$table = NicepageHelpersNicepage::getSectionsTable();
$table->load(array('page_id' => $row->id));
$props = $table->getProps();

$params = NicepageHelpersNicepage::getConfig($props['isPreview']);
$props = NicepageHelpersNicepage::prepareProps($props, $params, $context, $row, $rowParams);

$pageId         = $props['pageId'];
$bodyClass      = $props['bodyClass'];
$bodyStyle      = $props['bodyStyle'];
$head           = $props['head'];
$fonts          = $props['fonts'];
$publishHtml    = $props['publishHtml'];

$backlink       = $props['backlink'];
$pageCssUsedIds = $props['pageCssUsedIds'];
$hideHeader     = $props['hideHeader'];
$hideFooter     = $props['hideFooter'];
$hideBackToTop  = $props['hideBackToTop'];

$metaTags       = $props['metaTags'];
$customHeadHtml = $props['customHeadHtml'];
$metaGeneratorContent = $props['metaGeneratorContent'];
$canonical = $props['canonical'];

if ($metaTags) {
    $doc->addCustomTag($metaTags);
}

if ($customHeadHtml) {
    $doc->addCustomTag($customHeadHtml);
}

if ($metaGeneratorContent) {
    $doc->setMetaData('generator', $metaGeneratorContent);
}

if (!$canonical) {
    $canonical = JUri::getInstance()->toString();
}

JHtml::_('bootstrap.framework');

$backToTop = '';
if (isset($params['backToTop']) && !$hideBackToTop) {
    $backToTop = $params['backToTop'];
}

$publishCookiesSection = '';
$cookiesConfirmCode = '';
if (isset($params['cookiesConsent'])) {
    $cookiesConsent = json_decode($params['cookiesConsent'], true);
    if ($cookiesConsent && (!$cookiesConsent['hideCookies'] || $cookiesConsent['hideCookies'] === 'false')) {
        $publishCookiesSection = $cookiesConsent['publishCookiesSection'];
        $cookiesConfirmCode = $cookiesConsent['cookieConfirmCode'];
    }
}

$publishHtml = NicepageHelpersNicepage::processSectionsHtml($publishHtml, true, $pageId);
$css = NicepageHelpersNicepage::buildSiteStyleCss($params, $pageCssUsedIds, $publishHtml, $pageId);

$publishPageParts = str_replace(
    '[[site_path_live]]',
    JURI::root(),
    array(
        'publishHtml'   => $publishHtml,
        'head'          => $head,
        'bodyStyle'     => $bodyStyle,
        'header'        => $params['header'],
        'footer'        => $params['footer'],
        'publishCookiesSection' => $publishCookiesSection,
        'fonts' => $fonts,
    )
);

$publishHeader = '';
if ($publishPageParts['header'] && !$hideHeader) {
    $headerItem = json_decode($publishPageParts['header'], true);
    if ($headerItem) {
        ob_start();
        echo $headerItem['styles'];
        echo $headerItem['php'];
        $publishHeader = ob_get_clean();
        $publishHeader = NicepageHelpersNicepage::processSectionsHtml($publishHeader, true, 'header');
    }
}

$publishFooter = '';
if ($publishPageParts['footer'] && !$hideFooter) {
    $footerItem = json_decode($publishPageParts['footer'], true);
    if ($footerItem) {
        ob_start();
        echo $footerItem['styles'];
        echo $footerItem['php'];
        $publishFooter = ob_get_clean();
        $publishFooter = NicepageHelpersNicepage::processSectionsHtml($publishFooter, true, 'footer');
    }
}

$assets =  $doc->baseurl . '/components/com_nicepage/assets';
?>
<!DOCTYPE html>
<html>
<head>
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <link rel="canonical" href="<?php echo $canonical; ?>">
    <link rel="stylesheet" type="text/css" media="all" href="<?php echo $assets; ?>/css/nicepage.css" id="nicepage-style-css">
    <link rel="stylesheet" type="text/css" media="all" href="<?php echo $assets; ?>/css/media.css" id="theme-media-css">
    <jdoc:include type="head" />
    <?php if (isset($params['jquery']) && $params['jquery'] == '1') : ?>
        <script type="text/javascript" src="<?php echo $assets; ?>/js/jquery.js"></script>
    <?php endif; ?>
    <script type="text/javascript" src="<?php echo $assets; ?>/js/nicepage.js"></script>
    <?php echo $publishPageParts['fonts']; ?>
    <style>
        <?php echo $css; ?>
        <?php echo $publishPageParts['head']; ?>
    </style>
    <?php echo $cookiesConfirmCode; ?>
</head>
<body class="<?php echo $bodyClass; ?>" style="<?php echo $publishPageParts['bodyStyle']; ?>">
<?php
echo $publishHeader;
?>
<?php echo $publishPageParts['publishHtml']; ?>
<?php
echo $publishFooter
?>
<?php echo $backlink; ?>
<?php echo $editLinkHtml; ?>
<?php echo $publishPageParts['publishCookiesSection']; ?>
<?php echo $backToTop; ?>
</body>
</html>