<?php
/**
 * @package Nicepage Website Builder
 * @author Nicepage https://www.nicepage.com
 * @copyright Copyright (c) 2016 - 2019 Nicepage
 * @license http://www.gnu.org/licenses/gpl-2.0.html GNU/GPLv2 or later
 */
defined('_JEXEC') or die;

/**
 * Class PlgContentNicepage
 */
class PlgContentNicepage extends JPlugin
{

    private $_isNicepageTheme = '0';
    private $_sectionsHtml = '';
    private $_backlink = '';
    private $_cookiesConsent = '';
    private $_cookiesConfirmCode = '';
    private $_backToTop = '';
    private $_canonicalUrl = '';
    private $_styles = '';
    private $_scripts = '';
    private $_config = null;
    private $_originalName = 'nicepage';

    /**
     * @param object    $context Context
     * @param object    $row     Row
     * @param JRegistry $params  Parameters
     * @param int       $page    Page number
     */
    public function onContentPrepare($context, &$row, &$params, $page = 0)
    {
        $app = JFactory::getApplication();
        $originalName = $this->_originalName;

        if ($app->isAdmin() || !$this->_npInstalled() || (property_exists($row, 'doubleСall') && $row->doubleСall)) {
            return;
        }

        if ($context !== 'com_content.article' && $context !== 'com_content.featured' && $context !== 'com_content.category') {
            return;
        }

        $isBlog = $context === 'com_content.featured' || $context === 'com_content.category';

        if (isset($row->id)) {
            $componentPath = dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage';
            JLoader::register('NicepageHelpersNicepage', $componentPath . '/helpers/nicepage.php');

            $this->_config = NicepageHelpersNicepage::getConfig();
            $table = NicepageHelpersNicepage::getSectionsTable();
            if ($table->load(array('page_id' => $row->id))) {
                if ($isBlog && isset($table->props['introImgStruct']) && $table->props['introImgStruct']) {
                    $row->pageIntroImgStruct = json_decode($this->_fixImagePaths($table->props['introImgStruct']), true);
                    return;
                }

                $row->{$originalName} = true;
                $theme = $app->getTemplate(true);
                $this->_isNicepageTheme = $theme->params->get($originalName . 'theme', '0');

                $props = $table->getProps();
                $pageView = isset($props['pageView']) ? $props['pageView'] : 'landing';
                if ($pageView === 'landing') {
                    $row->introtext = $row->text = "<!--np_landing-->" .
                        base64_encode(serialize(array('context' => $context, 'row' => $row, 'params' => $params, 'editLinkHtml' => $this->_getEditLinkHtml($params, $row->id)))) .
                        "<!--/np_landing-->";
                } else {
                    $props = NicepageHelpersNicepage::prepareProps($props, $this->_config, $context, $row, $params);
                    $this->_addPageProperties($props);
                    $this->_setScripts();
                    $this->_setStyles($props);
                    $this->_setBacklink($props, $pageView);
                    $this->_setSectionsHtml($props);
                    $this->_setCookiesConsent($props);
                    $this->_setBackToTop($props);
                    $this->_setCanonicalUrl($props);
                    $row->introtext = $row->text = "<!--np_content-->" . $this->_getSectionsHtml() . $this->_getEditLinkHtml($params, $row->id) . "<!--/np_content-->\n" .
                        "<!--np_styles-->" . $this->_getStyles() . "<!--/np_styles-->\n" .
                        "<!--np_scripts-->" . $this->_getScripts() . "<!--/np_scripts-->\n" .
                        "<!--np_backlink-->" . $this->_getBacklink() . "<!--/np_backlink-->\n" .
                        "<!--np_cookies-->" . $this->_getCookiesConsent() . "<!--/np_cookies-->\n" .
                        "<!--np_cookiesconfirmcode-->" . $this->_getCookiesConfirmCode() . "<!--/np_cookiesconfirmcode-->\n" .
                        "<!--np_backtotop-->" . $this->_getBackToTop() . "<!--/np_backtotop-->\n" .
                        "<!--np_canonical-->" . $this->_getCanonicalUrl() . "<!--/np_canonical-->\n" .
                        "<!--np_view-->" . $pageView . "<!--/np_view-->\n";
                }
            }
        }
        return;
    }

    /**
     * Get edit link html
     *
     * @param JRegistry $params Parameters
     * @param int       $id     Article id
     *
     * @return string
     */
    private function _getEditLinkHtml($params, $id) {
        $html = '';
        $adminUrl = JURI::root(true) . '/administrator';
        $icon = dirname($adminUrl) . '/components/com_nicepage/assets/images/button-icon.png?r=' . md5(mt_rand(1, 100000));
        $link = $adminUrl . '/index.php?option=com_nicepage&task=nicepage.autostart&postid=' . $id;
        if ($params->get('access-edit')) {
            $html= <<<HTML
        <a href="$link" target="_blank" class="edit-nicepage-button">Edit Page</a>
        <style>
            a.edit-nicepage-button {
                position: fixed;
                top: 0;
                right: 0;
                background: url($icon) no-repeat 5px 6px;
                background-size: 16px;
                color: #4184F4;
                font-family: Georgia;
                margin: 10px;
                display: inline-block;
                padding: 5px 5px 5px 25px;
                font-size: 14px;
                line-height: 18px;
                background-color: #fff;
                border-radius: 3px;
                border: 1px solid #eee;
                z-index: 9999;
                text-decoration: none;
            }
            a.edit-nicepage-button:hover {
                color: #BC5A5B;
            }
        </style>
HTML;
        }
        return $html;
    }

    /**
     * Check on exist nicepage plugin
     *
     * @return bool
     */
    private function _npInstalled()
    {
        if (!file_exists(dirname(JPATH_PLUGINS) . '/administrator/components/com_nicepage')) {
            return false;
        }
        if (!JComponentHelper::getComponent('com_nicepage', true)->enabled) {
            return false;
        }
        return true;
    }

    /**
     * Set sections html
     *
     * @param array $props Page properties
     */
    private function _setSectionsHtml($props)
    {
        $this->_sectionsHtml = NicepageHelpersNicepage::processSectionsHtml($props['publishHtml'], true, $props['pageId']);

        $autoResponsive = isset($this->_config['autoResponsive']) ? !!$this->_config['autoResponsive'] : true;
        if ($autoResponsive && $this->_isNicepageTheme == '0') {
            $responsiveScript = <<<SCRIPT
<script>
    (function ($) {
        var ResponsiveCms = window.ResponsiveCms;
        if (!ResponsiveCms) {
            return;
        }
        ResponsiveCms.contentDom = $('script:last').parent();
        
        if (typeof ResponsiveCms.recalcClasses === 'function') {
            ResponsiveCms.recalcClasses();
        }
    })(jQuery);
</script>
SCRIPT;
            $this->_sectionsHtml = $responsiveScript . $this->_sectionsHtml;
        }

        if ($this->_isNicepageTheme === '0') {
            $this->_sectionsHtml = '<div class="nicepage-container"><div style="' . $props['bodyStyle'] . '" class="'. $props['bodyClass'] .'">' . $this->_sectionsHtml . '</div></div>';
        } else {
            $bodyScript = <<<SCRIPT
<script>
var body = document.body;
    
    body.className += " {$props['bodyClass']}";
    body.style.cssText += " {$props['bodyStyle']}";
</script>
SCRIPT;
            $this->_sectionsHtml = $bodyScript . $this->_sectionsHtml;
        }
    }

    /**
     * Get sections html
     *
     * @return string
     */
    private function _getSectionsHtml()
    {
        return $this->_sectionsHtml;
    }

    /**
     * Set page backlink
     *
     * @param array  $props    Page properties
     * @param string $pageView Template type for page
     */
    private function _setBacklink($props, $pageView)
    {
        $backlink = $props['backlink'];
        if ($backlink && ($pageView == 'default' || $pageView === 'landing_with_header_footer')) {
            if ($this->_isNicepageTheme !== '1') {
                $backlink = '<div class="nicepage-container"><div class="'. $props['bodyClass'] .'">' . $backlink . '</div></div>';
            } else {
                $backlink = '';
            }
        }
        $this->_backlink = $backlink;
    }

    /**
     * Process backlink in content
     *
     * @param sring $content Content
     *
     * @return mixed
     */
    private function _processBackLink($content) {
        if (!$this->_config) {
            return $content;
        }
        $hideBacklink = isset($this->_config['hideBacklink']) ? (bool)$this->_config['hideBacklink'] : false;
        return $hideBacklink ? str_replace('u-backlink', 'u-backlink u-hidden', $content) : $content;
    }

    /**
     * Get page backlink
     *
     * @return string
     */
    private function _getBacklink()
    {
        return $this->_backlink;
    }

    /**
     * Set backtotop in content
     *
     * @param array $props Page properties
     */
    private function _setBackToTop($props) {
        $hideBackToTop = $props['hideBackToTop'];
        if (isset($this->_config['backToTop']) && !$hideBackToTop) {
            $this->_backToTop = '<div class="nicepage-container"><div class="'. $props['bodyClass'] .'">' . $this->_config['backToTop'] . '</div></div>';
        }
    }

    /**
     * Get page backlink
     *
     * @return string
     */
    private function _getBackToTop()
    {
        return $this->_backToTop;
    }

    /**
     * Set canonical url
     *
     * @param array $props Page properties
     */
    private function _setCanonicalUrl($props) {
        $this->_canonicalUrl = $props['canonical'];

    }

    /**
     * @return string
     */
    private function _getCanonicalUrl() {
        return $this->_canonicalUrl;
    }

    /**
     * Set page cookies consent
     *
     * @param array $props Props
     */
    private function _setCookiesConsent($props)
    {
        if ($this->_isNicepageTheme === '1') {
            return;
        }

        if (isset($this->_config['cookiesConsent'])) {
            $cookiesConsent = json_decode($this->_config['cookiesConsent'], true);
            if ($cookiesConsent && (!$cookiesConsent['hideCookies'] || $cookiesConsent['hideCookies'] === 'false')) {
                $this->_cookiesConsent = '<div class="nicepage-container"><div class="'. $props['bodyClass'] .'">' . $this->_fixImagePaths($cookiesConsent['publishCookiesSection']) . '</div></div>';
                $this->_cookiesConfirmCode = $cookiesConsent['cookieConfirmCode'];
            }
        }
    }

    /**
     * Get page cookies consent
     *
     * @return string
     */
    private function _getCookiesConsent()
    {
        return $this->_cookiesConfirmCode;
    }

    /**
     * Get page cookies confirm code
     *
     * @return string
     */
    private function _getCookiesConfirmCode()
    {
        return $this->_cookiesConsent;
    }

    /**
     * Add custom page properties
     *
     * @param array $props Page properties
     */
    private function _addPageProperties($props)
    {
        $document = JFactory::getDocument();
        if ($props['metaTags']) {
            $document->addCustomTag($props['metaTags']);
        }
        if ($props['customHeadHtml']) {
            $document->addCustomTag($props['customHeadHtml']);
        }
        if ($props['metaGeneratorContent']) {
            $document->setMetaData('generator', $props['metaGeneratorContent']);
        }
    }

    /**
     * Set plugin styles
     *
     * @param array $props Page properties
     */
    private function _setStyles($props)
    {
        $pageCssUsedIds = $props['pageCssUsedIds'];
        $siteStyleCss = NicepageHelpersNicepage::buildSiteStyleCss($this->_config, $pageCssUsedIds, $props['publishHtml'], $props['pageId']);

        $sectionsHead = $props['head'];
        $autoResponsive = isset($this->_config['autoResponsive']) ? !!$this->_config['autoResponsive'] : true;
        $assets = JURI::root(true) . '/components/com_nicepage/assets';
        if ($autoResponsive && $this->_isNicepageTheme == '0') {
            $sectionsHead = preg_replace('#\/\*RESPONSIVE_MEDIA\*\/([\s\S]*?)\/\*\/RESPONSIVE_MEDIA\*\/#', '', $sectionsHead);
            $this->_styles .= '<link href="' . $assets . '/css/responsive.css" rel="stylesheet">';
        } else {
            $sectionsHead = preg_replace('#\/\*RESPONSIVE_CLASS\*\/([\s\S]*?)\/\*\/RESPONSIVE_CLASS\*\/#', '', $sectionsHead);
            if ($this->_isNicepageTheme == '0') {
                $this->_styles .= '<link href="' . $assets . '/css/media.css" rel="stylesheet">';
            }
        }
        $dynamicCss = $siteStyleCss . $sectionsHead;
        if ($this->_isNicepageTheme !== '1') {
            $this->_styles .= '<link href="' . $assets . '/css/page-styles.css" rel="stylesheet">';
            $dynamicCss = $this->wrapStyles($dynamicCss);
        }
        $this->_styles .= $props['fonts'];
        $this->_styles .= '<style id="nicepage-style-css">' . $dynamicCss . '</style>';
    }

    /**
     * Wrap styles by container
     *
     * @param string $dynamicCss Additional styles
     *
     * @return null|string|string[]
     */
    public function wrapStyles($dynamicCss)
    {
        return preg_replace_callback(
            '/([^{}]+)\{[^{}]+?\}/',
            function ($match) {
                $selectors = $match[1];
                $parts = explode(',', $selectors);
                $newSelectors = implode(
                    ',',
                    array_map(
                        function ($part) {
                            if (!preg_match('/html|body|sheet|keyframes/', $part)) {
                                return ' .nicepage-container ' . $part;
                            } else {
                                return $part;
                            }
                        },
                        $parts
                    )
                );
                return str_replace($selectors, $newSelectors, $match[0]);
            },
            $dynamicCss
        );
    }

    /**
     * Get plugin styles
     *
     * @return string
     */
    private function _getStyles()
    {
        return $this->_styles;
    }

    /**
     * Set plugin scripts
     */
    private function _setScripts()
    {
        if ($this->_isNicepageTheme !== '1') {
            $assets = JURI::root(true) . '/components/com_nicepage/assets';
            if (isset($this->_config['jquery']) && $this->_config['jquery'] == '1') {
                $this->_scripts .= '<script src="' . $assets . '/js/jquery.js"></script>';
            }
            $this->_scripts .= '<script src="' . $assets . '/js/nicepage.js"></script>';
        }
    }

    /**
     * Get plugin scripts
     *
     * @return string
     */
    private function _getScripts()
    {
        return $this->_scripts;
    }

    /**
     * Fix all image paths
     *
     * @param string $content Html content
     *
     * @return mixed
     */
    private function _fixImagePaths($content) {
        return str_replace('[[site_path_live]]', JURI::root(), $content);
    }
}